from langchain.output_parsers import StructuredOutputParser, ResponseSchema
from langchain.prompts import PromptTemplate
from langchain_openai import ChatOpenAI
from .config import OPENAI_API_KEY

# Define the expected response schema
response_schemas = [
    ResponseSchema(name="amount", description="The amount of cryptocurrency to transfer."),
    ResponseSchema(name="currency", description="The cryptocurrency (BTC or mUSD)."),
    ResponseSchema(name="recipient", description="The recipient's wallet address."),
]

# Initialize the structured output parser
output_parser = StructuredOutputParser.from_response_schemas(response_schemas)

# Instantiate the ChatOpenAI LLM
llm = ChatOpenAI(temperature=0, openai_api_key=OPENAI_API_KEY)

# Prepare the prompt template
prompt_template = PromptTemplate(
    template="Extract transaction details from this request:\n{input}\n{format_instructions}",
    input_variables=["input"],
    partial_variables={"format_instructions": output_parser.get_format_instructions()},
)

def extract_transaction_details(prompt: str):
    """
    Uses LLM to extract structured transaction details from user input.
    """
    formatted_prompt = prompt_template.format(input=prompt)
    response = llm.invoke(formatted_prompt)

    try:
        return output_parser.parse(response.content)
    except Exception as e:
        return f"❌ Failed to extract transaction details: {str(e)}"